import sqlalchemy as sa


class RBDMS:
    _engine = None
    _connection = None

    @property
    def engine(self):
        return self._engine

    @property
    def connection(self):
        return self._connection

    def __init__(self, connection_str: str):
        if connection_str is None or connection_str == '':
            raise Exception('Variable connection_str not provided')

        self._engine = sa.create_engine(connection_str)
        self._connection = self._engine.connect()

    def __del__(self):
        self.connection.close()
        self._engine.dispose()



import json
from os.path import join
from lazyutils.config.Configuration import Config
from lazyutils.secrets.Secrets import Secrets


class LocalSecrets(Secrets):
    _config = None
    _folder = None
    _file = None

    def _load(self, path: str):
        self._config = Config(path)
        self._folder = self._config['Secrets']['folder']
        self._file = self._config['Secrets']['file']

        filepath = join(self._folder, self._file)
        with open(filepath, 'r') as f:
            self._secrets = json.load(f)



from enum import Enum, auto

from lazyutils.persistance.Local import LocalLayerStorage


class PersistanceLayer(Enum):
    LOCAL = auto()
    AWS_S3 = auto()
    RBDMS = auto()


# TODO Implement unit tests
def PersistanceFactory(layer: PersistanceLayer = PersistanceLayer.LOCAL, path: str = ''):

    if layer is PersistanceLayer.LOCAL:
        return LocalLayerStorage(path)
    else:
        raise NotImplementedError




import os
import tempfile

from lazyutils.random.Randomize import random_string


class TempWritableFile:
    file_path: str
    """
    The tempfile library doesn't work well on others platforms like Windows. So, for you to create a file that 
    is accessible for another file descriptor on your process, you need to really create the file.
    This class create the file and delete it when gc is called.
    """
    file_mode: str

    def __init__(self, file_prefix: str, mode: str = 'w'):
        dir_name = tempfile.gettempdirb()
        file_name = file_prefix + random_string(8)
        self.file_path = os.path.join(dir_name.decode(), file_name)
        self.file_mode = mode

    def __enter__(self):
        self.fd = open(self.file_path, self.file_mode)
        return self

    def __exit__(self, type, value, traceback):
        self.fd.close()

    def __del__(self):
        if not self.fd.closed:
            self.fd.close()
        os.remove(self.file_path)

    def write(self, s: str):
        self.fd.write(s)

    def flush(self):
        self.fd.flush()



import json
import os
from abc import ABC
from os import listdir
from os.path import isfile, join, exists
import re
import pandas as pd

from lazyutils.config.Configuration import Config
from lazyutils.misc.Dates import now_strftime
from lazyutils.persistance.Persistance import Persistance


class LocalLayerStorage(Persistance, ABC):

    _localstorage = '.localdata'
    _layer = 'layer'
    _processed = 'processed'

    def __init__(self, layerfolder: str):
        super().__init__()
        self.config = Config()

        if self.config.has_option('Core', 'layers_base_path'):
            self._localstorage = self.config['Core']['layers_base_path']

        if not exists(self._localstorage):
            os.mkdir(self._localstorage)

        self._layer = join(os.getcwd(), self._localstorage, layerfolder)

        if not exists(self._layer):
            os.mkdir(self._layer)

        if self.config.has_option('Core', 'layers_base_path'):
            self._processed = self.config['Core']['processed_posfix_folder']

        self._processed = join(os.getcwd(), self._localstorage, layerfolder+self._processed)

        if not exists(self._processed):
            os.mkdir(self._processed)

    def _save_dataframe(self, prefix: str, df: pd.DataFrame):
        jsonobj = json.loads(df.to_json(orient='records'))
        self._save_dict(prefix, jsonobj)

    def _save_dict(self, prefix: str, obj: dict):
        filename = join(self._layer, prefix) + '_' + now_strftime(fmt='%Y-%m-%d_%H_%M_%S') + '.json'

        with open(filename, "w") as jfile:
            json.dump(obj, jfile)

    def _save_list(self, prefix: str, obj: list):
        filename = join(self._layer, prefix) + '_' + now_strftime(fmt='%Y-%m-%d_%H_%M_%S') + '.json'

        with open(filename, "w") as f:
            json.dump(obj, f, ensure_ascii=False)

    def getallfileslist(self, query: str) -> list:
        return [f for f in listdir(self._layer) if isfile(join(self._layer, f)) and re.match(query, f)]

    def getallfiles(self, query: str) -> list:

        onlyfiles = [f for f in listdir(self._layer) if isfile(join(self._layer, f)) and re.match(query, f)]
        jlist = []

        for f in onlyfiles:
            with open(join(self._layer, f)) as file:
                jlist.append(file.read())

        return jlist

    def countallfiles(self, query: str) -> int:
        onlyfiles = [f for f in listdir(self._layer) if isfile(join(self._layer, f)) and re.match(query, f)]

        return len(onlyfiles)

    def getfilecontent(self, file: str):
        filepath = join(self._layer, file)
        with open(filepath, 'r') as f:
            return f.read()

    def mark_as_processed_files(self, files: list):
        for f in files:
            os.replace(
                join(self._layer, f),
                join(self._processed, f)
            )



from enum import Enum, auto

from lazyutils.secrets.LocalSecrets import LocalSecrets


class Secrets(Enum):
    LOCAL = auto()
    AWS_SECRETS_MANAGER = auto()


def SecretsFactory(vault: Secrets, path=''):
    if vault == Secrets.LOCAL:
        local = LocalSecrets()
        local.load()
        return local
    else:
        raise NotImplementedError



from typing import List

import pandas as pd


class Persistance:
    _layer = None

    def save(self, prefix: str, karg):
        return self._save_map[type(karg)](prefix, karg)

    def _save_dict(self, prefix: str, obj: dict):
        raise NotImplementedError

    def _save_list(self, prefix: str, obj: list):
        raise NotImplementedError

    def _save_dataframe(self, prefix: str, df: pd.DataFrame):
        raise NotImplementedError

    def get(self, query: str):
        raise NotImplementedError

    def getallfileslist(self, query: str) -> list:
        raise NotImplementedError

    def getallfiles(self, query: str) -> list:
        raise NotImplementedError

    def countallfiles(self, query: str) -> int:
        raise NotImplementedError

    def getfilecontent(self, file: str):
        raise NotImplementedError

    def mark_as_processed_files(self, files: List[str]):
        raise NotImplementedError

    def __init__(self):
        self._save_map = {
            pd.DataFrame: self._save_dataframe,
            dict: self._save_dict,
            list: self._save_list
        }



import configparser
import logging
import os

from lazyutils.structure.Singleton import Singleton


class Configuration(Singleton):
    config = None
    _log_level: str = 'DEBUG'
    config_path: str

    def __init__(self, config_file_path: str = './config/example-config.ini'):
        if self.config is not None:
            return

        self.config_path = config_file_path
        self.config = configparser.ConfigParser()
        sections_read = self.config.read(config_file_path)

        if len(sections_read) > 0:
            self._log_level = self.config['LOG']['log-level']

        logging.basicConfig(
            format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
            datefmt='%d/%m/%Y %H:%M:%S',
            level=self._log_level
        )

        logging.getLogger().addHandler(logging.StreamHandler())

        logging.debug('== Configuration ==')
        for k in list(self.config.keys()):
            logging.debug('%s: %s', k, self.config[k])

    def check(self, keys_to_check: list) -> list:
        not_found = []
        flattened_config = [item for sublist in self.config for item in sublist]
        for k in keys_to_check:
            if k not in flattened_config:
                not_found.append(k)

        return not_found


def Config(filepath: str = ''):
    c = Configuration() if filepath == '' else Configuration(filepath)
    return c.config


def ConfigFromEnv():
    base_path = os.getenv("CONFIG_PATH") if type(os.getenv("CONFIG_PATH")) is None else os.sep
    config_path = os.path.join(base_path, 'config', 'config.ini')
    return Config(config_path)



import logging
from datetime import datetime, timedelta


# TODO Implement unit tests
from typing import List


def deltatime_to_strftime(delta: int, fmt: str = '%Y-%m-%d %H:%M:%S') -> str:
    dt = datetime.now() - timedelta(days=delta)
    return dt.strftime(fmt)


def now_strftime(fmt: str = '%Y-%m-%d %H:%M:%S') -> str:
    return datetime.now().strftime(fmt)


def conditional_strptime(tm: str, formats: List[str], ignoreerrors: bool = True) -> datetime:

    for fm in formats:
        try:
            return datetime.strptime(tm, fm)
        except ValueError:
            logging.debug(f'Format {fm} not valid for string {tm}')

    if ignoreerrors:
        return datetime.now()
    else:
        raise Exception('Not possible to format datetime string')



import setuptools
from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='lazydev-utils',
    packages=setuptools.find_packages(),
    version='0.1.3',
    description='Lazy utils with broad different function to improve python implementation',
    author='Paulo Born',
    license='MIT',
    install_requires=['pandas==1.1.4'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest==6.1.2'],
    test_suite='tests',
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    py_modules=["lazyutils"]
)



import threading


class Singleton(object):
    __instances = {}
    __lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        with cls.__lock:  # If used with many threads, may impact on object creation performance
            if cls not in cls.__instances:
                cls.__instances[cls] = super(Singleton, cls).__new__(cls)
        return cls.__instances[cls]

    @classmethod
    def reset(cls):
        """
        Reset class instance from internal control, but you should rebuild again, so garbage collector can dispose
        current instance. \n
        \n
        foo = Foo() \n
        foo.reset() \n
        foo = Foo() #If not, you are going to keep using the same memory instance \n

        """

        with cls.__lock:
            cls.__instances.pop(cls)
